package org.springframework.test.context;

package ..deTemp.1733323053328;

import java.lang.annotation.Annotation;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.function.Predicate;
import org.springframework.core.SpringProperties;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.annotation.MergedAnnotationCollectors;
import org.springframework.core.annotation.MergedAnnotations;
import org.springframework.lang.Nullable;
import org.springframework.test.context.NestedTestConfiguration;
import org.springframework.util.Assert;
import org.springframework.util.ClassUtils;
import org.springframework.util.ConcurrentLruCache;
import org.springframework.util.ObjectUtils;

public abstract class TestContextAnnotationUtils {
  private static final ConcurrentLruCache<Class<?>, NestedTestConfiguration.EnclosingConfiguration> cachedEnclosingConfigurationModes = new ConcurrentLruCache(32, org.springframework.test.context.TestContextAnnotationUtils::lookUpEnclosingConfiguration);
  
  @Nullable
  private static volatile NestedTestConfiguration.EnclosingConfiguration defaultEnclosingConfigurationMode;
  
  public static boolean hasAnnotation(Class<?> clazz, Class<? extends Annotation> annotationType) {
    return (findMergedAnnotation(clazz, annotationType) != null);
  }
  
  @Nullable
  public static <T extends Annotation> T findMergedAnnotation(Class<?> clazz, Class<T> annotationType) {
    return findMergedAnnotation(clazz, annotationType, org.springframework.test.context.TestContextAnnotationUtils::searchEnclosingClass);
  }
  
  @Nullable
  private static <T extends Annotation> T findMergedAnnotation(Class<?> clazz, Class<T> annotationType, Predicate<Class<?>> searchEnclosingClass) {
    AnnotationDescriptor<T> descriptor = findAnnotationDescriptor(clazz, annotationType, searchEnclosingClass, new HashSet<>());
    return (descriptor != null) ? (T)descriptor.getAnnotation() : null;
  }
  
  public static <T extends Annotation> Set<T> getMergedRepeatableAnnotations(Class<?> clazz, Class<T> annotationType) {
    Set<T> mergedAnnotations = (Set<T>)MergedAnnotations.from(clazz, MergedAnnotations.SearchStrategy.INHERITED_ANNOTATIONS).stream(annotationType).collect(MergedAnnotationCollectors.toAnnotationSet());
    if (!mergedAnnotations.isEmpty())
      return mergedAnnotations; 
    if (searchEnclosingClass(clazz))
      return getMergedRepeatableAnnotations(clazz.getEnclosingClass(), annotationType); 
    return Collections.emptySet();
  }
  
  @Nullable
  public static <T extends Annotation> AnnotationDescriptor<T> findAnnotationDescriptor(Class<?> clazz, Class<T> annotationType) {
    Assert.notNull(annotationType, "Annotation type must not be null");
    return findAnnotationDescriptor(clazz, annotationType, org.springframework.test.context.TestContextAnnotationUtils::searchEnclosingClass, new HashSet<>());
  }
  
  @Nullable
  private static <T extends Annotation> AnnotationDescriptor<T> findAnnotationDescriptor(@Nullable Class<?> clazz, Class<T> annotationType, Predicate<Class<?>> searchEnclosingClass, Set<Annotation> visited) {
    if (clazz == null || Object.class == clazz)
      return null; 
    if (AnnotationUtils.isAnnotationDeclaredLocally(annotationType, clazz))
      return new AnnotationDescriptor(clazz, clazz.getAnnotation(annotationType)); 
    AnnotationDescriptor<T> descriptor = null;
    for (Annotation composedAnn : clazz.getDeclaredAnnotations()) {
      Class<? extends Annotation> composedType = composedAnn.annotationType();
      if (!AnnotationUtils.isInJavaLangAnnotationPackage(composedType.getName()) && visited.add(composedAnn)) {
        descriptor = findAnnotationDescriptor(composedType, annotationType, searchEnclosingClass, visited);
        if (descriptor != null)
          return new AnnotationDescriptor(clazz, descriptor.getDeclaringClass(), descriptor.getAnnotation()); 
      } 
    } 
    for (Class<?> ifc : clazz.getInterfaces()) {
      descriptor = findAnnotationDescriptor(ifc, annotationType, searchEnclosingClass, visited);
      if (descriptor != null)
        return new AnnotationDescriptor(clazz, descriptor.getDeclaringClass(), descriptor.getAnnotation()); 
    } 
    descriptor = findAnnotationDescriptor(clazz.getSuperclass(), annotationType, searchEnclosingClass, visited);
    if (descriptor != null)
      return descriptor; 
    if (searchEnclosingClass.test(clazz)) {
      descriptor = findAnnotationDescriptor(clazz.getEnclosingClass(), annotationType, searchEnclosingClass, visited);
      if (descriptor != null)
        return descriptor; 
    } 
    return null;
  }
  
  @Nullable
  public static UntypedAnnotationDescriptor findAnnotationDescriptorForTypes(Class<?> clazz, Class<? extends Annotation>... annotationTypes) {
    assertNonEmptyAnnotationTypeArray((Class<?>[])annotationTypes, "The list of annotation types must not be empty");
    return findAnnotationDescriptorForTypes(clazz, annotationTypes, new HashSet<>());
  }
  
  @Nullable
  private static UntypedAnnotationDescriptor findAnnotationDescriptorForTypes(@Nullable Class<?> clazz, Class<? extends Annotation>[] annotationTypes, Set<Annotation> visited) {
    if (clazz == null || Object.class == clazz)
      return null; 
    for (Class<? extends Annotation> annotationType : annotationTypes) {
      if (AnnotationUtils.isAnnotationDeclaredLocally(annotationType, clazz))
        return new UntypedAnnotationDescriptor(clazz, clazz.getAnnotation((Class)annotationType), (Class[])annotationTypes); 
    } 
    for (Annotation composedAnnotation : clazz.getDeclaredAnnotations()) {
      if (!AnnotationUtils.isInJavaLangAnnotationPackage(composedAnnotation) && visited.add(composedAnnotation)) {
        UntypedAnnotationDescriptor untypedAnnotationDescriptor = findAnnotationDescriptorForTypes(composedAnnotation
            .annotationType(), annotationTypes, visited);
        if (untypedAnnotationDescriptor != null)
          return new UntypedAnnotationDescriptor(clazz, untypedAnnotationDescriptor.getDeclaringClass(), untypedAnnotationDescriptor
              .getAnnotation(), (Class[])annotationTypes); 
      } 
    } 
    for (Class<?> ifc : clazz.getInterfaces()) {
      UntypedAnnotationDescriptor untypedAnnotationDescriptor = findAnnotationDescriptorForTypes(ifc, annotationTypes, visited);
      if (untypedAnnotationDescriptor != null)
        return new UntypedAnnotationDescriptor(clazz, untypedAnnotationDescriptor.getDeclaringClass(), untypedAnnotationDescriptor
            .getAnnotation(), (Class[])annotationTypes); 
    } 
    UntypedAnnotationDescriptor descriptor = findAnnotationDescriptorForTypes(clazz.getSuperclass(), annotationTypes, visited);
    if (descriptor != null)
      return descriptor; 
    if (searchEnclosingClass(clazz)) {
      descriptor = findAnnotationDescriptorForTypes(clazz.getEnclosingClass(), annotationTypes, visited);
      if (descriptor != null)
        return descriptor; 
    } 
    return null;
  }
  
  public static boolean searchEnclosingClass(Class<?> clazz) {
    return (ClassUtils.isInnerClass(clazz) && 
      getEnclosingConfiguration(clazz) == NestedTestConfiguration.EnclosingConfiguration.INHERIT);
  }
  
  static void clearCaches() {
    cachedEnclosingConfigurationModes.clear();
    defaultEnclosingConfigurationMode = null;
  }
  
  private static NestedTestConfiguration.EnclosingConfiguration getEnclosingConfiguration(Class<?> clazz) {
    return (NestedTestConfiguration.EnclosingConfiguration)cachedEnclosingConfigurationModes.get(clazz);
  }
  
  private static NestedTestConfiguration.EnclosingConfiguration lookUpEnclosingConfiguration(Class<?> clazz) {
    Predicate<Class<?>> searchEnclosingClass = ClassUtils::isInnerClass;
    NestedTestConfiguration nestedTestConfiguration = findMergedAnnotation(clazz, NestedTestConfiguration.class, searchEnclosingClass);
    return (nestedTestConfiguration != null) ? nestedTestConfiguration.value() : getDefaultEnclosingConfigurationMode();
  }
  
  private static NestedTestConfiguration.EnclosingConfiguration getDefaultEnclosingConfigurationMode() {
    NestedTestConfiguration.EnclosingConfiguration defaultConfigurationMode = defaultEnclosingConfigurationMode;
    if (defaultConfigurationMode == null) {
      String value = SpringProperties.getProperty("spring.test.enclosing.configuration");
      NestedTestConfiguration.EnclosingConfiguration enclosingConfigurationMode = NestedTestConfiguration.EnclosingConfiguration.from(value);
      defaultConfigurationMode = (enclosingConfigurationMode != null) ? enclosingConfigurationMode : NestedTestConfiguration.EnclosingConfiguration.INHERIT;
      defaultEnclosingConfigurationMode = defaultConfigurationMode;
    } 
    return defaultConfigurationMode;
  }
  
  private static void assertNonEmptyAnnotationTypeArray(Class<?>[] annotationTypes, String message) {
    if (ObjectUtils.isEmpty((Object[])annotationTypes))
      throw new IllegalArgumentException(message); 
    for (Class<?> clazz : annotationTypes) {
      if (!Annotation.class.isAssignableFrom(clazz))
        throw new IllegalArgumentException("Array elements must be of type Annotation"); 
    } 
  }
}
